@@ -22,6 +22,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #pragma once
 
+#include "defs.h"
 #include <the_Foundation/hash.h>
 #include <the_Foundation/ptrarray.h>
 #include <the_Foundation/string.h>
@@ -31,7 +32,7 @@ iDeclareType(GmRequest)
 
 iDeclareType(Bookmark)
 iDeclareTypeConstruction(Bookmark)
-    
+
 /* These values are not serialized as-is in bookmarks.ini. Instead, they are included in `tags`
    with a dot prefix. This helps retain backwards and forwards compatibility. */
 enum iBookmarkFlags {
@@ -48,8 +49,12 @@ enum iBookmarkFlags {
 struct Impl_Bookmark {
     iHashNode node;
     iString   url;
+    iString   originalUrl; /* prior to updating via permanent redirect */
     iString   title;
     iString   tags;
+    iString   notes;    /* free-form comments */
+    iString   identity; /* if not empty, the identity (fingerprint) to activate when
+                           opening the bookmark */
     uint32_t  flags;
     iChar     icon;
     iTime     when;
@@ -60,26 +65,8 @@ struct Impl_Bookmark {
 iLocalDef uint32_t  id_Bookmark         (const iBookmark *d) { return d->node.key; }
 iLocalDef iBool     isFolder_Bookmark   (const iBookmark *d) { return isEmpty_String(&d->url); }
 
-iBool   hasParent_Bookmark  (const iBookmark *, uint32_t parentId);
-int     depth_Bookmark      (const iBookmark *);
-
-//iBool   hasTag_Bookmark     (const iBookmark *, const char *tag);
-//void    addTag_Bookmark     (iBookmark *, const char *tag);
-//void    removeTag_Bookmark  (iBookmark *, const char *tag);
-
-//iLocalDef void addTagIfMissing_Bookmark(iBookmark *d, const char *tag) {
-//    if (!hasTag_Bookmark(d, tag)) {
-//        addTag_Bookmark(d, tag);
-//    }
-//}
-//iLocalDef void addOrRemoveTag_Bookmark(iBookmark *d, const char *tag, iBool add) {
-//    if (add) {
-//        addTagIfMissing_Bookmark(d, tag);
-//    }
-//    else {
-//        removeTag_Bookmark(d, tag);
-//    }
-//}
+iBool   hasParent_Bookmark              (const iBookmark *, uint32_t parentId);
+int     depth_Bookmark                  (const iBookmark *);
 
 int     cmpTitleAscending_Bookmark      (const iBookmark **, const iBookmark **);
 int     cmpTree_Bookmark                (const iBookmark **, const iBookmark **);
@@ -97,23 +84,29 @@ typedef int   (*iBookmarksCompareFunc)  (const iBookmark **, const iBookmark **)
 void        clear_Bookmarks             (iBookmarks *);
 void        load_Bookmarks              (iBookmarks *, const char *dirPath);
 void        save_Bookmarks              (const iBookmarks *, const char *dirPath);
+void        serialize_Bookmarks         (const iBookmarks *, iStream *outs);
+void        deserialize_Bookmarks       (iBookmarks *, iStream *ins, enum iImportMethod);
 
 uint32_t    add_Bookmarks               (iBookmarks *, const iString *url, const iString *title,
                                          const iString *tags, iChar icon);
+uint32_t    addToFolder_Bookmarks       (iBookmarks *, const iString *url, const iString *title,
+                                         const iString *tags, iChar icon, uint32_t folderId);
 iBool       remove_Bookmarks            (iBookmarks *, uint32_t id);
 iBookmark * get_Bookmarks               (iBookmarks *, uint32_t id);
 void        reorder_Bookmarks           (iBookmarks *, uint32_t id, int newOrder);
-iBool       updateBookmarkIcon_Bookmarks(iBookmarks *, const iString *url, iChar icon);
+iBool       updateIcons_Bookmarks       (iBookmarks *, const iString *url, iChar icon);
+iBool       updateUrls_Bookmark         (iBookmarks *, const iString *oldUrl, const iString *newUrl);
 void        setRecentFolder_Bookmarks   (iBookmarks *, uint32_t folderId);
 void        sort_Bookmarks              (iBookmarks *, uint32_t parentId, iBookmarksCompareFunc cmp);
+
 void        fetchRemote_Bookmarks       (iBookmarks *);
 void        requestFinished_Bookmarks   (iBookmarks *, iGmRequest *req);
 
 iChar       siteIcon_Bookmarks          (const iBookmarks *, const iString *url);
 uint32_t    findUrl_Bookmarks           (const iBookmarks *, const iString *url); /* O(n) */
+uint32_t    findUrlIdent_Bookmarks      (const iBookmarks *, const iString *url, const iString *identFp); /* O(n) */
 uint32_t    recentFolder_Bookmarks      (const iBookmarks *);
 
-//iBool   filterTagsRegExp_Bookmarks      (void *regExp, const iBookmark *);
 iBool       filterHomepage_Bookmark     (void *, const iBookmark *);
 
 /**
