@@ -26,7 +26,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "gmrequest.h"
 #include "app.h"
 
-#if defined (iPlatformMsys)
+#if defined (iPlatformMsys) || defined (iPlatformWindows)
 #   include "win32.h"
 #endif
 
@@ -41,8 +41,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/stringlist.h>
 #include <the_Foundation/toml.h>
 
-const char *mimeType_FontPack = "application/lagrange-fontpack+zip";
-
 float scale_FontSize(enum iFontSize size) {
     static const float sizes[max_FontSize] = {
         1.000, /* UI sizes */
@@ -618,6 +616,9 @@ static const iString *userFontsDirectory_Fonts_(const iFonts *d) {
 
 void init_Fonts(const char *userDir) {
     iFonts *d = &fonts_;
+    if (isTerminal_Platform()) {
+        return; /* fonts not needed */
+    }
     d->indexPattern = new_RegExp(":([0-9]+)$", 0);    
     initCStr_String(&d->userDir, userDir);
     const iString *userFontsDir = userFontsDirectory_Fonts_(d);
@@ -631,7 +632,7 @@ void init_Fonts(const char *userDir) {
         setReadOnly_FontPack(pack, iTrue);
         loadArchive_FontPack(pack, archive_Resources()); /* should never fail if we've made it this far */
         pushBack_PtrArray(&d->packs, pack);
-#if defined (iPlatformMsys)
+#if defined (iPlatformMsys) || defined (iPlatformWindows)
         /* The system UI font is used as the default font. */
         iString *winPath = collect_String(windowsDirectory_Win32());
         iString *segoePath = collect_String(concatCStr_Path(winPath, "Fonts\\segoeui.ttf"));
@@ -794,6 +795,9 @@ void init_Fonts(const char *userDir) {
 
 void deinit_Fonts(void) {
     iFonts *d = &fonts_;
+    if (isTerminal_Platform()) {
+        return; /* fonts are not used */
+    }
     unloadFonts_Fonts_(d);
     iAssert(isEmpty_ObjectList(d->files));
     deinit_PtrArray(&d->specOrder);
