@@ -36,14 +36,15 @@ void    setupWindow_iOS         (iWindow *window);
 iBool   processEvent_iOS        (const SDL_Event *);
 void    playHapticEffect_iOS    (enum iHapticEffect effect);
 void    exportDownloadedFile_iOS(const iString *path);
-void    pickFileForOpening_iOS  (void);
 void    pickFile_iOS            (const char *command); /* ` path:%s` will be appended */
 void    openTextActivityView_iOS(const iString *text);
 void    openFileActivityView_iOS(const iString *path);
+iBool   openUri_iOS             (const iString *uri);
 
 iBool   isPhone_iOS             (void);
 void    safeAreaInsets_iOS      (float *left, float *top, float *right, float *bottom);
 int     displayRefreshRate_iOS  (void);
+float   displayScale_iOS        (const iWindow *window);
 
 /*----------------------------------------------------------------------------------------------*/
 
