@@ -26,6 +26,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "command.h"
 #include "labelwidget.h"
 #include "listwidget.h"
+#include "uploadwidget.h"
+#include "misfin.h"
 #include "../gmcerts.h"
 #include "../app.h"
 
@@ -39,6 +41,7 @@ struct Impl_CertItem {
     uint32_t  id;
     int       indent;
     iChar     icon;
+    iBool     isMisfin;
     iBool     isBold;
     iString   label;
     iString   meta;
@@ -46,10 +49,11 @@ struct Impl_CertItem {
 
 void init_CertItem(iCertItem *d) {
     init_ListItem(&d->listItem);
-    d->id     = 0;
-    d->indent = 0;
-    d->icon   = 0;
-    d->isBold = iFalse;
+    d->id       = 0;
+    d->indent   = 0;
+    d->icon     = 0;
+    d->isMisfin = iFalse;
+    d->isBold   = iFalse;
     init_String(&d->label);
     init_String(&d->meta);
 }
@@ -87,6 +91,7 @@ static iGmIdentity *menuIdentity_CertListWidget_(const iCertListWidget *d) {
 }
 
 static void updateContextMenu_CertListWidget_(iCertListWidget *d) {
+    iAssert(d->contextItem);
     iArray *items = collectNew_Array(sizeof(iMenuItem));
     const iString *docUrl = url_DocumentWidget(document_App());
     size_t firstIndex = 0;
@@ -94,11 +99,13 @@ static void updateContextMenu_CertListWidget_(iCertListWidget *d) {
         pushBack_Array(items, &(iMenuItem){ format_CStr("```%s", cstr_String(docUrl)) });
         firstIndex = 1;
     }
+    /* Used URLs. */
+    const iGmIdentity *ident = menuIdentity_CertListWidget_(d);
     const iMenuItem ctxItems[] = {
         { person_Icon " ${ident.use}", 0, 0, "ident.use arg:1" },
         { close_Icon " ${ident.stopuse}", 0, 0, "ident.use arg:0" },
         { close_Icon " ${ident.stopuse.all}", 0, 0, "ident.use arg:0 clear:1" },
-        { "---", 0, 0, NULL },
+        { "---" },
         { edit_Icon " ${menu.edit.notes}", 0, 0, "ident.edit" },
         { "${ident.fingerprint}", 0, 0, "ident.fingerprint" },
 #if defined (iPlatformAppleDesktop)
@@ -108,19 +115,17 @@ static void updateContextMenu_CertListWidget_(iCertListWidget *d) {
         { magnifyingGlass_Icon " ${menu.reveal.filemgr}", 0, 0, "ident.reveal" },
 #endif
         { export_Icon " ${ident.export}", 0, 0, "ident.export" },
-        { "---", 0, 0, NULL },
+        { "---" },
         { delete_Icon " " uiTextCaution_ColorEscape "${ident.delete}", 0, 0, "ident.delete confirm:1" },
     };
     pushBackN_Array(items, ctxItems, iElemCount(ctxItems));
-    /* Used URLs. */
-    const iGmIdentity *ident = menuIdentity_CertListWidget_(d);
     if (ident) {
         size_t insertPos = firstIndex + 3;
         if (!isEmpty_StringSet(ident->useUrls)) {
-            insert_Array(items, insertPos++, &(iMenuItem){ "---", 0, 0, NULL });
+            insert_Array(items, insertPos++, &(iMenuItem){ "---" });
         }
         iBool usedOnCurrentPage = iFalse;
-        iConstForEach(StringSet, i, ident->useUrls) {            
+        iConstForEach(StringSet, i, ident->useUrls) {
             const iString *url = i.value;
             usedOnCurrentPage |= startsWithCase_String(docUrl, cstr_String(url));
             iRangecc urlStr = range_String(url);
@@ -140,9 +145,19 @@ static void updateContextMenu_CertListWidget_(iCertListWidget *d) {
         else {
             remove_Array(items, firstIndex);
         }
+        if (d->contextItem->isMisfin) {
+            if (d->contextItem->isMisfin) {
+                insertN_Array(
+                    items,
+                    0,
+                    (iMenuItem[]){ { envelope_Icon " ${ident.sendmsg}", 0, 0, "ident.sendmsg" },
+                                   { "---" } },
+                    2);
+            }
+        }
     }
-    destroy_Widget(d->menu);    
-    d->menu = makeMenu_Widget(as_Widget(d), data_Array(items), size_Array(items));    
+    destroy_Widget(d->menu);
+    d->menu = makeMenu_Widget(as_Widget(d), data_Array(items), size_Array(items));
 }
 
 static void itemClicked_CertListWidget_(iCertListWidget *d, iCertItem *item, size_t itemIndex) {
@@ -157,7 +172,7 @@ static void itemClicked_CertListWidget_(iCertListWidget *d, iCertItem *item, siz
         updateContextMenu_CertListWidget_(d);
         arrange_Widget(d->menu);
         openMenuFlags_Widget(d->menu,
-                             bounds_Widget(w).pos.x < mid_Rect(rect_Root(w->root)).x
+                             bounds_Widget(w).pos.x < mid_Rect(rect_Root(w->root)).x /* on left side? */
                                  ? topRight_Rect(itemRect_ListWidget(&d->list, itemIndex))
                                  : addX_I2(topLeft_Rect(itemRect_ListWidget(&d->list, itemIndex)),
                                            -width_Widget(d->menu)),
@@ -180,7 +195,7 @@ static iBool processEvent_CertListWidget_(iCertListWidget *d, const SDL_Event *e
             return iTrue;
         }
         else if (isCommand_Widget(w, ev, "ident.use")) {
-            iGmIdentity *ident = menuIdentity_CertListWidget_(d);            
+            iGmIdentity *ident = menuIdentity_CertListWidget_(d);
             const iString *tabUrl = urlQueryStripped_String(url_DocumentWidget(document_App()));
             if (ident) {
                 if (argLabel_Command(cmd, "clear")) {
@@ -217,7 +232,16 @@ static iBool processEvent_CertListWidget_(iCertListWidget *d, const SDL_Event *e
             if (ident) {
                 const iString *fps = collect_String(
                     hexEncode_Block(collect_Block(fingerprint_TlsCertificate(ident->cert))));
-                SDL_SetClipboardText(cstr_String(fps));
+                if (isTerminal_Platform()) {
+                    makeMessage_Widget(
+                        "${ident.fingerprint}",
+                        cstr_String(fps),
+                        (iMenuItem[]){ { "${dlg.message.ok}", SDLK_RETURN, 0, "message.ok" } },
+                        1);
+                }
+                else {
+                    SDL_SetClipboardText(cstr_String(fps));
+                }
             }
             return iTrue;
         }
@@ -236,6 +260,13 @@ static iBool processEvent_CertListWidget_(iCertListWidget *d, const SDL_Event *e
             }
             return iTrue;
         }
+        else if (isCommand_UserEvent(ev, "contextkey") && isFocused_Widget(w)) {
+            const size_t cursor = d->list.cursorItem;
+            if (cursor != iInvalidPos) {
+                itemClicked_CertListWidget_(d, item_ListWidget(&d->list, cursor), cursor);
+            }
+            return iTrue;
+        }
         else if (isCommand_Widget(w, ev, "ident.setnotes")) {
             iGmIdentity *ident = pointerLabel_Command(cmd, "ident");
             if (ident) {
@@ -244,7 +275,11 @@ static iBool processEvent_CertListWidget_(iCertListWidget *d, const SDL_Event *e
             }
             return iTrue;
         }
-        else if (isCommand_Widget(w, ev, "ident.pickicon")) {
+        else if (isCommand_Widget(w, ev, "ident.sendmsg")) {
+            const iGmIdentity *ident = menuIdentity_CertListWidget_(d);
+            if (ident && isMisfin_GmIdentity(ident)) {
+                openMessageComposer_Misfin(NULL, ident);
+            }
             return iTrue;
         }
         else if (isCommand_Widget(w, ev, "ident.reveal")) {
@@ -303,7 +338,7 @@ static iBool processEvent_CertListWidget_(iCertListWidget *d, const SDL_Event *e
                 invalidateItem_ListWidget(&d->list, d->contextIndex);
             }
             d->contextIndex = hoverItemIndex_ListWidget(&d->list);
-            updateContextMenu_CertListWidget_(d);                
+            updateContextMenu_CertListWidget_(d);
             /* TODO: Some callback-based mechanism would be nice for updating menus right
                before they open? At least move these to `updateContextMenu_ */
             const iGmIdentity *ident  = constHoverIdentity_CertListWidget(d);
@@ -365,7 +400,7 @@ static void draw_CertItem_(const iCertItem *d, iPaint *p, iRect itemRect,
         bg = uiBackgroundUnfocusedSelection_ColorId;
         fillRect_Paint(p, itemRect, bg);
     }
-//    iInt2 pos = itemRect.pos; 
+//    iInt2 pos = itemRect.pos;
     const int fg = isHover ? (isPressing ? uiTextPressed_ColorId : uiTextFramelessHover_ColorId)
                            : uiTextStrong_ColorId;
     const iBool isUsedOnDomain = (d->indent != 0);
@@ -436,6 +471,7 @@ iBool updateItems_CertListWidget(iCertListWidget *d) {
         item->id = (uint32_t) index_PtrArrayConstIterator(&i);
         item->icon = 0x1f464; /* person */
         set_String(&item->label, name_GmIdentity(ident));
+        item->isMisfin = isMisfin_GmIdentity(ident);
         iDate until;
         validUntil_TlsCertificate(ident->cert, &until);
         const iBool isActive = isUsedOn_GmIdentity(ident, tabUrl);
@@ -449,12 +485,27 @@ iBool updateItems_CertListWidget(iCertListWidget *d) {
             ident->flags & temporary_GmIdentityFlag
                 ? cstr_Lang("ident.temporary")
                 : cstrCollect_String(format_Date(&until, cstr_Lang("ident.expiry")));
+        if (item->isMisfin) {
+            //iString *addr = misfinIdentity_GmIdentity(ident, &item->meta);
+            //set_String(&item->label, addr);
+            //delete_String(addr);
+            item->icon = 0x1f4e7; /* envelope */
+            clear_String(&item->meta);
+            // set_String(&item->meta, &ident->notes);
+            // appendFormat_String(&item->meta, "\n%s", expiry);
+        }
         if (isEmpty_String(&ident->notes)) {
-            appendFormat_String(&item->meta, "\n%s", expiry);
+            if (!isEmpty_String(&item->meta)) {
+                appendCStr_String(&item->meta, "\n");
+            }
+            appendFormat_String(&item->meta, "%s", expiry);
         }
         else {
+            if (!isEmpty_String(&item->meta)) {
+                appendCStr_String(&item->meta, " \u2014 ");
+            }
             appendFormat_String(&item->meta,
-                                " \u2014 %s\n%s%s",
+                                "%s\n%s%s",
                                 expiry,
                                 escape_Color(uiHeading_ColorId),
                                 cstr_String(&ident->notes));
